class TuyaCIMonitor
	attr_accessor :methods, :name

	ALWAYS_OPEN = 1
	ALWAYS_CLOSE = 2
	AUTO = 3

	def initialize
		# puts 'in initialize'
		@methods = {}
	end

	def exe_action(action, options, key)
		method = @methods[action.to_sym]
		begin
			strategy_method = @methods[:strategy]
			strategy, strategy_key = strategy_method.call options if strategy_method

			unless strategy.class == Fixnum
				strategy = ALWAYS_OPEN
			end
			trigger_call method, action, options, key, strategy, strategy_key
		rescue Exception => e
			if e.class == TuyaCIMonitorStopError
				puts "Trigger Exception cached, tuya_stop_build has been used. the stop message is: #{$!}".red
				raise $!
			else
				puts "Trigger Exception cached, you can puts detail in your error method".red
				exe_error action, $!, $@, options
			end

		ensure
		end
	end

	def trigger_call(method, action, options, key, strategy, strategy_key)

		call_strategy = false
		if strategy == ALWAYS_CLOSE
			call_strategy = false
		elsif strategy == ALWAYS_OPEN
			call_strategy = true
		elsif strategy == AUTO
			monitors = TuyaCIDSL::TuyaDSL.instance
			if monitors.strategy_auto.has_key? strategy_key
				call_strategy = true
				options[:strategy] = monitors.strategy_auto[strategy_key]
			end
		end

		if call_strategy
			if method
				puts "Trigger '#{action}' in '#{key}'".green
				method.call options
			end
		end
	end

	def strategy_always_open
		ALWAYS_OPEN
	end

	def strategy_always_close
		ALWAYS_CLOSE
	end

	def strategy_auto
		AUTO
	end


	def tuya_stop_build(message)
		raise TuyaCIMonitorStopError, message
	end
	
	def tuya_sh(shell)
		dsl = TuyaCIDSL::DSLExecute.new shell
		dsl.execute
	end

	def tuya_sh_from_git(shell, git)
		dsl = TuyaCIDSL::DSLExecute.new shell
		dsl.download git
		dsl.execute
	end

	def exe_error(process, error, position, options)
		method = @methods['error'.to_sym]
		# require 'tuya/ci/DSL/exe/dsl_exe'
		method.call error, position, options, process if method
	end

	def exe
		@methods.each_key do |key|
			l = @methods[key]
			l.call
		end
	end

	def ci(*args, &block)
		# puts "in ci args is: #{args}"
		dsl 'root', args, &block
	end

	def dsl(process, args, &block)
		# puts "in dsl process is: #{process} args is: #{args}"
		if block_given?
			if process
				# puts "process: #{process}"
				if process == 'root'
					instance_eval &block
				else
					@methods[process] = block
				end
			end
		end
	end

	def method_missing(m, *args, &block)
		# puts "in method_missing method is: #{m}, args is: #{args}"
		dsl m, args, &block
	end
end

class TuyaCIMonitorStopError < StandardError

	attr_reader :object

	# def initialize(object)
	# 	@object = object
	# end

	# begin
	# 	raise TuyaCIMonitorStopError.new("an object"), "a message"
	# rescue TuyaCIMonitorStopError => e
	# 	puts e.message # => "a message"
	# 	puts e.object # => "an object"
	# end

end

class TuyaCIMonitorMethods
	attr_accessor :name, :args, :block
	def initialize
		@name = ''
		@args = ''
		@block = nil
	end
end
