module TuyaCIDSL

	require 'singleton'

	class TuyaDSL

		attr_accessor :strategy_auto

		include Singleton

		def initialize
			@default_root_file = 'CIFile'
			@default_folder = 'CIMonitors'
			@files = []
			@monitors = {}
			@strategy_auto = {}
			load_files
		end

		def load_monitors_git(git)
			git_downloader = TYCiCore::GitDownloader.new git
			git_downloader.download '.dsl_common'
			load_monitors_path git_downloader.folder_path
		end

		def load_monitors_path(path)
			if File.exist? path
				@files += Dir["#{path}/**/*"].reject{|o|File.directory?(o)}
			end
		end

		def insert_strategy(strategy)

			unless strategy.class == Hash
				return
			end

			strategy.keys.each do |item|
				unless item.is_a?(Symbol)
					strategy[item.to_sym] = strategy[item]
				end
			end
			@strategy_auto = strategy
		end

		def load_monitors
			@files.each do |item|
				add_monitor item
			end
			# puts @monitors
		end

		def trigger_module_begin(options)
			trigger 'before_module_build', options
		end

		def trigger_module_end(options)
			trigger 'after_module_build', options
		end

		def trigger_sdk_begin(options)
			trigger 'before_sdk_build', options
		end

		def trigger_sdk_end(options)
			trigger 'after_sdk_build', options
		end

		def trigger_project_begin(options)
			trigger 'before_project_build', options
		end

		def trigger_project_pod_update_end(options)
			trigger 'building_project_pod_update_end', options
		end

		def trigger_project_end(options)
			trigger 'after_project_build', options
		end

		private


		# trigger 执行策略
		def trigger(action, options)
			# puts "trigger action: #{action} options: #{options}"
			@monitors.each_key do |key|
				item = @monitors[key]
				item.exe_action action, options, key
			end

		end

		def load_files
			@files << "./#{@default_root_file}" if File.exist? "./#{@default_root_file}"
			if File.exist? "./#{@default_folder}/"
				@files += Dir["./#{@default_folder}/**/*"].reject{|o|File.directory?(o)}
			end
		end

		def add_monitor(file)
			analyzer = TuyaCIDSL::Analyzer.new file
			monitor = analyzer.p_monitor
			@monitors[file] = monitor
		end
	end

end